@@ -8,7 +8,6 @@ import requests
 import semantic_version
 from notebook.base.handlers import IPythonHandler
 from notebook.utils import url_path_join
-from notebook import notebookapp
 
 from jupyter_cogram import __version__
 
@@ -42,7 +41,10 @@ def save_token(token: Text, loc: Path = config_location) -> None:
     if not loc.is_dir():
         loc.mkdir(parents=True)
 
-    (loc / token_file_name).write_text(token)
+    p = loc / token_file_name
+
+    logger.debug(f"Saving token '{token}' at path '{p}'.")
+    p.write_text(token)
 
 
 def fetch_token(loc: Path = config_location) -> Optional[Text]:
@@ -65,7 +67,7 @@ def post_auth_token(token: Text) -> requests.Response:
 
 
 def is_installation_up_to_date(
-    pypi_url: Text = package_pypi_url,
+        pypi_url: Text = package_pypi_url,
 ) -> bool:
     logger.info("Checking if local installation is up to date.")
 
@@ -101,12 +103,12 @@ def is_installation_up_to_date(
         return False
 
 
-def fetch_suggestion_codex(
-    queries,
-    cell_contents: Optional[List[Text]] = None,
-    kernel_id: Optional[Text] = None,
-    session_id: Optional[Text] = None,
-    auth_token: Optional[Text] = None,
+def fetch_suggestion_code(
+        queries,
+        cell_contents: Optional[List[Text]] = None,
+        kernel_id: Optional[Text] = None,
+        session_id: Optional[Text] = None,
+        auth_token: Optional[Text] = None,
 ):
     url = f"{backend_url}/{completions_enpdoint}"
     res = requests.post(
@@ -120,8 +122,6 @@ def fetch_suggestion_codex(
         },
     )
 
-    save_token(auth_token)
-
     if res.status_code >= 400:
         if res.json() and res.json().get("error") is not None:
             msg = res.json()["error"]
@@ -156,25 +156,31 @@ class JupyterCogramHandler(IPythonHandler):
         auth_token = data.get("auth_token", "'")
 
         try:
-            suggestions = fetch_suggestion_codex(
+            logger.debug("Submitting to API server")
+            suggestions = fetch_suggestion_code(
                 queries, cell_contents, kernel_id, session_id, auth_token
             )
 
             logger.debug(f"Have suggestions\n{json.dumps(suggestions, indent=2)}")
             if "choices" in suggestions and suggestions["choices"]:
-                message = [c["text"] for c in suggestions["choices"]]
+                message = [c.get("text", "") for c in suggestions["choices"]]
             else:
                 message = None
 
-            # logger.debug(f"Extracted suggestion:\n{suggestion}")
             status = "success"
             status_code = 200
-
         except Exception as e:
-            error_dict = json.loads(str(e))
-            status = error_dict["status"]
-            message = error_dict["error_msg"]
-            status_code = error_dict["status_code"]
+            logger.exception("Encountered error when submitting to API server.")
+            try:
+                error_dict = json.loads(str(e))
+                status = error_dict["status"]
+                message = error_dict["error_msg"]
+                status_code = error_dict["status_code"]
+            except Exception:
+                logger.exception("Unknown error.")
+                status_code = 400
+                status = "Error"
+                message = "Unknown error."
 
         response = {"status": status, "message": message}
         self.set_status(status_code)
@@ -284,10 +290,10 @@ def load_jupyter_server_extension(nb_server_app):
         [
             (url_path_join(web_app.settings["base_url"], uri), handler)
             for uri, handler in [
-                ("/cogram", JupyterCogramHandler),
-                ("/token", JupyterCogramTokenHandler),
-                ("/checkVersion", JupyterCogramVersionHandler),
-            ]
+            ("/cogram", JupyterCogramHandler),
+            ("/token", JupyterCogramTokenHandler),
+            ("/checkVersion", JupyterCogramVersionHandler),
+        ]
         ],
     )
     logger.debug("loaded_jupyter_server_extension: jupyter-cogram")
